/**
 * @file
 * Combine Lists implementation.
 *
 * @copyright Copyright © 2012–2020 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

    VuoModuleMetadata({
                          "title" : "Combine Lists",
                          "keywords" : [ "sort", "indecies", "big", "high", "small", "most", ">", "<", "upper", "limit", "bound", "range" ],
                          "version" : "2.1.0",
                          "genericTypes" : {
                              "VuoGenericType1" : {
                                  "defaultType" : "VuoReal",
                                  "compatibleTypes" : [ "VuoInteger", "VuoReal", "VuoPoint2d", "VuoPoint3d", "VuoPoint4d", "VuoImage", "VuoLayer" ]
                              }
                          },
                          "node": {
                              "exampleCompositions" : ["CombineLists.vuo"]
                          }
                      });

    void nodeEvent
    (                                                                                                               //Main setup
            VuoInputData(VuoList_VuoGenericType1) list1,                                                            //Setup input list 1
            VuoInputData(VuoList_VuoGenericType1) list2,                                                            //Setup input list 2
            VuoInputData(VuoInteger, {"default":1,"suggestedMin":1}) itemCount1,                                    //Setup integer input for the batch size of list 1
            VuoInputData(VuoInteger, {"default":1,"suggestedMin":1}) itemCount2,                                    //Setup integer input for the batch size of list 2
            VuoOutputData(VuoList_VuoGenericType1) combinedList                                                     //Setup the output list
    )                                                                                                               //Main setup Done

    {                                                                                                               //Function start
        *combinedList = VuoListCreate_VuoGenericType1();                                                            //Create the output list and point it to the output

        unsigned long list1Count = floor((VuoListGetCount_VuoGenericType1(list1) / itemCount1)) + itemCount1;       //Count the batches for list 1
        unsigned long list2Count = floor((VuoListGetCount_VuoGenericType1(list2) / itemCount2)) + itemCount2;       //Count the batches for list 2

        for (unsigned long i = 0, j = 0; i <= list1Count && j <= list2Count; i+=itemCount1, j+=itemCount2)          //Create a double for loop iterating over the batches for both input lists
            {
            if(i<=list1Count){                                                                                      //If the batch count is less than or equal to the size of list 1
                for(unsigned long k = 1; k <= itemCount1; ++k){                                                     //Iterate over the list 1 batch size
                    VuoListAppendValue_VuoGenericType1(                                                             //Append a value to a list
                                *combinedList,                                                                      //Point to the output list
                                VuoListGetValue_VuoGenericType1(                                                    //The value to append, get a value from a list
                                    list1,                                                                          //Get the value from list 1
                                    i + k)                                                                          //The index to get the value from, batch size + batch index
                                );
                        }                                                                                           //For loop done
            }                                                                                                       //If statement done

            if(j <= list2Count){                                                                                    //If the batch count is less than or equal to the size of list 1
                for(unsigned long m = 1; m <= itemCount2; ++m){                                                     //Iterate over the list 2 batch size
                    VuoListAppendValue_VuoGenericType1(                                                             //Append a value to a list
                                *combinedList,                                                                      //Point to the output list
                                VuoListGetValue_VuoGenericType1(                                                    //The value to append, get a value from a list
                                    list2,                                                                          //Get the value from list 2
                                    j + m)                                                                          //The index to get the value from, batch size + batch index
                                );
                    }                                                                                               //For loop done
        }                                                                                                           //If statement done
}                                                                                                                   //For loop done
}                                                                                                                   //Function done
