/**
 * @file
 * Morph List implementation.
 *
 * @copyright Copyright © 2012–2020 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleMetadata({
                     "title" : "Morph Point List",
                     "description" : "Morphs between two point lists",
                     "keywords" : [ "Morph", "Warp", "Lerp", "Interpolate"],
					 "version" : "1.0.0",
					 "dependencies" : [ ],
                      "genericTypes" : {
                          "VuoGenericType1" : {
                              "defaultType" : "VuoReal",
                              "compatibleTypes" : ["VuoReal", "VuoPoint2d", "VuoPoint3d", "VuoPoint4d"]
                          }
                      },
					 "node": {
                         "isInterface" : false,
                          "exampleCompositions" : ["MorphPointList.vuo" ]
					 }
				 });

void nodeEvent
(                                                                                                                   //Main setup
        VuoInputData(VuoReal, {"default": 0., "suggestedMin":0., "suggestedMax":1.0}) morph,                        //Setup VuoReal input for morph control
        VuoInputData(VuoList_VuoGenericType1) list1,                                                                //Setup an input for list 1
        VuoInputData(VuoList_VuoGenericType1) list2,                                                                //Setup an input for list 2
        VuoOutputData(VuoList_VuoGenericType1) morphList                                                            //Setup an output list
)                                                                                                                   //Main setup done

{                                                                                                                   //Function start
    *morphList = VuoListCreate_VuoGenericType1();                                                                   //Create the output list and point it to the output
    unsigned long list1Count = VuoListGetCount_VuoGenericType1(list1);                                              //Count the items in list 1
    unsigned long list2Count = VuoListGetCount_VuoGenericType1(list2);                                              //Count the items in list 2
    unsigned long maxCount = MAX(list1Count, list2Count);                                                           //Find the largest list

    for (unsigned long i = 1; i <= maxCount; ++i)                                                                   //Create for loop iterating over the longest list
        {
            VuoGenericType1 sum;                                                                                    //Create variable sum
            if (i <= list1Count && i <= list2Count){                                                                //If the iteration step is under the length of both lists
                sum = VuoGenericType1_add(                                                                          //Add
                            VuoGenericType1_multiply(VuoListGetValue_VuoGenericType1(list1, i), 1.0 - morph),       //Value from list 1 multiplied with (1 - the morph value)
                            VuoGenericType1_multiply(VuoListGetValue_VuoGenericType1(list2, i), morph)              //Value from list 2 multiplied with the morph value
                            );
            }
            else if (i <= list1Count){                                                                              //Else if the iteration step is under the length of list 1
                sum = VuoListGetValue_VuoGenericType1(list1, i);                                                    //Set sum to be the value at list 1
            }
            else {                                                                                                  //Else (implicit if the iteration step is larger than the length of list 1 but shorter than list 2)
                sum = VuoListGetValue_VuoGenericType1(list2, i);                                                    //Set sum to be the value at list 2
            }                                                                                                       //If/else done

            VuoListAppendValue_VuoGenericType1(*morphList, sum);                                                    //Append the value of sum to the output list
        }                                                                                                           //For loop done
}                                                                                                                   //Function done
