/**
 * @file
 * Scale point list to center implementation.
 *
 * @copyright Copyright © 2012–2020 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleMetadata({
                      "title" : "Scale List to Center",
                      "keywords" : [ "multiply", "product", "times", "*", "arithmetic", "calculate", "limit", "bounds" ],
                      "version" : "1.0.0",
                      "genericTypes" : {
                          "VuoGenericType1" : {
                              "compatibleTypes" : ["VuoReal", "VuoPoint2d", "VuoPoint3d", "VuoPoint4d"]
                          }
                      },
                      "node" : {
                          "exampleCompositions" : ["ScaleListToCenter.vuo" ]
                      }
                  });

void nodeEvent
(                                                                               //Main setup
        VuoInputData(VuoList_VuoGenericType1) list,                             //Setup a list input
        VuoInputData(VuoReal, {"default":1.0,"suggestedMin":0.0}) scaleMax,     //Setup a VuoReal input for the max scale amount
        VuoInputData(VuoReal, {"default":0.0,"suggestedMin":0.0}) scaleMin,     //Setup a VuoReal input for the minimum scale amount
        VuoInputData(VuoCurve, {"default":"linear"}) curve,                     //Setup a curve input
        VuoInputData(VuoCurveEasing, {"default":"in"}) easing,                  //setup an easing input
        VuoOutputData(VuoList_VuoGenericType1) scaledList                       //Setup an output list
)                                                                               //Setup done

{                                                                               //Function start
    *scaledList = VuoListCreate_VuoGenericType1();                              //Create a list and point it to the output

    unsigned long listCount = VuoListGetCount_VuoGenericType1(list);            //Get the length of the input list
    VuoReal duration = (listCount - 1.0)/2.0;                                   //Create a VuoReal variable duration set to half the length of the list

    for (unsigned long i = 1; i <= listCount; ++i)                              //Create a for loop iterating over the length of the input list
    {
        VuoGenericType1 point = VuoListGetValue_VuoGenericType1(list, i+1);     //Create a variable 'point' and set it to the value of the input list
        VuoGenericType1 scale = VuoGenericType1_curve(                          //Create a variable 'scale' and modulate it with the curve function
                    i - 1,                                                      //Time - set to the current iteration -1 to compensate for 0 base
                    scaleMin,                                                   //startPosition - set to input 'scaleMin'
                    scaleMax,                                                   //endPosition - set to input 'scaleMax'
                    duration,                                                   //duration - set to variable 'duration
                    curve,                                                      //curve - set to input 'curve'
                    easing,                                                     //easing - set to input 'easing'
                    VuoLoopType_Mirror                                          //loop - hardcoded to looptype mirror. In combination with an effective length of 2 in normalized terms as defined by the 'duration' variable, this will give two "turns" of the curve node where one will be mirrored
                    );                                                          //Curve function done

        VuoGenericType1 scaledPoint = VuoGenericType1_scale(point, scale);      //Create variable 'scaledPoint' with the value of 'point' scaled by the 'scale' variable. Note that using VuoGenericType1_multiply() in this case will hard crash Vuo here with any other value than VuoReal as you'll then try to fit a 2d/3d/4d point into a VuoReal.

        VuoListAppendValue_VuoGenericType1(*scaledList, scaledPoint);           //Point to the output list and append the value of 'scaledPoint'
    }                                                                           //For loop done
}                                                                               //Function done
