/**
 * @file
 * Scale Point List implementation
 *
 * @copyright Copyright © 2012–2020 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include "math.h"

VuoModuleMetadata({
                      "title" : "Scale Point List",
                      "keywords" : [ "magnitude", "bigger", "smaller", "some", "small", "most", ">", "<", "upper", "range" ],
                      "version" : "2.1.0",
                      "genericTypes" : {
                          "VuoGenericType1" : {
                              "defaultType" : "VuoReal",
                              "compatibleTypes" : ["VuoReal", "VuoPoint2d", "VuoPoint3d", "VuoPoint4d" ]
                          }
                      },
                      "node": {
                          "exampleCompositions" : ["ScalePointList.vuo"]
                      }
                  });

void nodeEvent
(                                                                                                               //Main setup
        VuoInputData(VuoList_VuoGenericType1) list,                                                             //Setup a source input list
        VuoInputData(VuoList_VuoReal) magnitudeList,                                                            //Setup an input list for modulation of the source list
        VuoInputData(VuoReal, {"default":0.0}) offset,                                                          //Setup a VuoReal input for modulation of the modulation list
        VuoInputData(VuoBoolean, {"default":FALSE}) invert,                                                     //Setup a VuoBoolean input for changing the sign of the output list
        VuoOutputData(VuoList_VuoGenericType1) pointList                                                        //Setup the output list
)                                                                                                               //Setup done

{                                                                                                               //Function start
    unsigned long listCount = VuoListGetCount_VuoGenericType1(list);                                            //Get the length of the source list
    *pointList = VuoListCreate_VuoGenericType1();                                                               //Create the output list and point it to the output

    for(int i = 1; i <= listCount; ++i){                                                                        //Create a for loop iterating over the length of the source list
        VuoGenericType1 currentPoint = VuoListGetValue_VuoGenericType1(list, i);                                //Create a variable 'currentPoint' and set it to the value of the source list at the current iteration

        VuoReal magnitude;                                                                                      //Create a VuoReal variable 'magnitude'
        if(invert == FALSE) {                                                                                   //If the boolean input 'invert' is set to false
            magnitude = VuoListGetValue_VuoReal(magnitudeList, i);                                              //variable 'magnitude' is set to the value in the 'magnitudeList' at the current iteration
        } else {                                                                                                //Else (implicit the boolean input 'invert' is true)
            magnitude = -VuoListGetValue_VuoReal(magnitudeList, i);                                             //variable 'magnitude is set to - (minus) the value in the 'magnitudeList' at the current iteration
        }                                                                                                       //If/Else done

        VuoGenericType1 newPoint = VuoGenericType1_multiply(currentPoint, VuoReal_add(magnitude, offset));      //Create variable 'newPoint' set to variable 'currentPoint' multiplied with the sum of variable 'magnitude' and input 'offset'

        VuoListAppendValue_VuoGenericType1(*pointList, newPoint);                                               //Point to the output list and append the value at variable 'newPoint'
    }                                                                                                           //For loop done
}                                                                                                               //Function done
