/**
 * @file
 * Sort by index implementation.
 *
 * @copyright Copyright © 2012–2018 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleMetadata({
                      "title" : "Shift List",
                      "keywords" : [ "sort", "indecies", "big", "high", "small", "most", ">", "<", "upper", "limit", "bound", "range" ],
                      "version" : "2.1.0",
                      "genericTypes" : {
                          "VuoGenericType1" : {
                              "defaultType" : "VuoReal",
                              "compatibleTypes" : [ "VuoInteger", "VuoReal", "VuoPoint2d", "VuoPoint3d", "VuoPoint4d", "VuoImage", "VuoLayer" ]
                          }
                      },
                      "node": {
                          "exampleCompositions" : ["ShiftList.vuo"]
                      }
                  });

void nodeEvent
(                                                                                   //Main setup
        VuoInputData(VuoList_VuoGenericType1) list,                                 //Setup an input list
        VuoInputData(VuoInteger, {"default":1,"suggestedMin":1}) startPosition,     //Setup a VuoInteger input
        VuoOutputData(VuoList_VuoGenericType1) reorderedList                        //Setup the output list
)                                                                                   //Main setupp done

{                                                                                   //Function start
    *reorderedList = VuoListCreate_VuoGenericType1();                               //Create the output list and point it to the output
    VuoInteger listSize = VuoListGetCount_VuoGenericType1(list);                    //Get the length of the input list

    for(int i = 0; i < listSize; i++){                                              //Create a for loop iterating over the 'listSize'
            VuoGenericType1 value = VuoListGetValue_VuoGenericType1(                //Create a variable 'value' and get it
                        list,                                                       //From the input list
                        (((startPosition - 1) + i) % listSize) + 1);                //At an index based on the input 'startPosition' + the current iteration, modulo of the 'listSize' corrected for 0-base
            VuoListAppendValue_VuoGenericType1(*reorderedList, value);              //Point to the output list and append the 'value'
        }                                                                           //For loop done
}                                                                                   //Function done
