/**
 * @file
 * Sort by index implementation.
 *
 * @copyright Copyright © 2012–2018 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleMetadata({
                      "title" : "Sort with index",
                      "keywords" : [ "sort", "indecies", "big", "high", "small", "most", ">", "<", "upper", "limit", "bound", "range" ],
                      "version" : "2.1.0",
                      "genericTypes" : {
                          "VuoGenericType1" : {
                              "defaultType" : "VuoReal",
                              "compatibleTypes" : [ "VuoInteger", "VuoReal" ]
                          }
                      },
                      "node": {
                          "exampleCompositions" : ["SortedIndex.vuo"]
                      }
                  });

void nodeEvent
(                                                                                           //Main setup
        VuoInputData(VuoList_VuoReal) values,                                               //Setup source input list
        VuoOutputData(VuoList_VuoReal, {"name":"Sorted"}) max,                              //Setup output list named 'Sorted' containing the sorted values from the source list
        VuoOutputData(VuoList_VuoInteger, {"name":"Index"}) position                        //Setup an output list containing the index numbers in the source list for the corresponding values in the sorted list
)                                                                                           //Setup done

{                                                                                           //Function start
    unsigned long valueCount = VuoListGetCount_VuoReal(values);                             //Count the items in the source list
    VuoList_VuoReal maxOut = VuoListCreate_VuoReal();                                       //Create a VuoReal list 'maxOut'
    VuoList_VuoInteger indexOut = VuoListCreate_VuoInteger();                               //Create a VuoInteger list 'indexOut'

    for(int i = 0; i < valueCount; ++i){                                                    //Create a for loop iterating over 'valueCount'
        VuoInteger pos;                                                                     //Create a VuoInteger value 'pos'
        VuoReal *termsArray = VuoListGetData_VuoReal(values);                               //Create a pointer to a C array containing the source list items
        VuoListInsertValue_VuoReal(maxOut, VuoReal_max(termsArray, valueCount, &pos), 0);   //Insert the maximum value from the 'termsArray' at the beginning of the 'maxOut' list and reference the 'pos' variable to the index it's at
        VuoListInsertValue_VuoInteger(indexOut, pos, 0);                                    //Insert the 'pos' value at the beginning of the 'indexOut' list
        VuoListSetValue_VuoReal(values, 0, pos, false);                                     //Set the current value in the 'values' list to 0 at index from 'pos'
    }                                                                                       //For loop done. This is probably a very, very, very crude and inefficient implementation of a sorting algorithm. It works by finding the largest number in the array along with its index and set it to 0 before finding the new maximum value.

    *max = VuoListSubset_VuoReal(maxOut, 1, valueCount);                                    //Create a new list with the values from 'maxOut' clamped to the 'valueCount' length and point it to the output list 'max' named 'Sorted'
    *position = VuoListSubset_VuoInteger(indexOut, 1, valueCount);                          //Create a new list with the values from 'indexOut' clamped to the 'valueCount' length and point it to the output list'position' named 'Index'
}                                                                                           //Function done
