/**
 * @file
 * Sort by index implementation.
 *
 * @copyright Copyright © 2012–2018 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleMetadata({
                      "title" : "Split XYZ List",
                      "keywords" : [ "sort", "indecies", "big", "high", "small", "most", ">", "<", "upper", "limit", "bound", "range" ],
                      "version" : "2.1.0",
                      "genericTypes" : {
                          "VuoGenericType1" : {
                              "defaultType" : "VuoReal",
                              "compatibleTypes" : [ "VuoInteger", "VuoReal", "VuoPoint2d", "VuoPoint3d", "VuoPoint4d", "VuoImage", "VuoLayer" ]
                          }
                      },
                      "node": {
                          "exampleCompositions" : ["SplitXYZList.vuo"]
                      }
                  });

void nodeEvent
(                                                                       //Main setup
        VuoInputData(VuoList_VuoPoint3d) list,                          //Create an input list
        VuoOutputData(VuoList_VuoReal) xList,                           //Create a VuoReal output list for the X component
        VuoOutputData(VuoList_VuoReal) yList,                           //Create a VuoReal output list for the Y component
        VuoOutputData(VuoList_VuoReal) zList                            //Create a VuoReal output list for the Z component
)                                                                       //Main setup done

{                                                                       //Function start
    *xList = VuoListCreate_VuoReal();                                   //Create a VuoReal list 'xList' and point it to the output list
    *yList = VuoListCreate_VuoReal();                                   //Create a VuoReal list 'yList' and point it to the output list
    *zList = VuoListCreate_VuoReal();                                   //Create a VuoReal list 'zList' and point it to the output list
    VuoInteger listSize = VuoListGetCount_VuoPoint3d(list);             //Count the items in the source list and store them in a VuoInteger 'listSize'

    for(int i = 1; i <= listSize; i++){                                 //Create a for loop iterating over the 'listSize' variable
            VuoPoint3d point = VuoListGetValue_VuoPoint3d(list, i);     //Create a VuoPoint3d variable 'point' and set it to the value of the source list at the current iteration
            VuoListAppendValue_VuoReal(*xList, point.x);                //Point to the 'xList' output list and append the x component of the current 'point' value
            VuoListAppendValue_VuoReal(*yList, point.y);                //Point to the 'yList' output list and append the y component of the current 'point' value
            VuoListAppendValue_VuoReal(*zList, point.z);                //Point to the 'zList' output list and append the z component of the current 'point' value
        }                                                               //For loop done
}                                                                       //Function done
